from django.shortcuts import render
from django.http.response import HttpResponse, HttpResponseBadRequest
from django.views.decorators.csrf import csrf_exempt  # 解除csrf验证
from wechat_sdk import WechatConf
from wechat_sdk import WechatBasic
from bs4 import BeautifulSoup
from myproject import settings
import random
import requests
import hashlib
import qrcode


def buy(request):
    return render(request, 'buy.html')


@csrf_exempt  # 去除csrf验证
def check_token(request):  # 服务器配置时验证token的函数
    conf = WechatConf(  # 实例化配置信息对象
        token='123456',  # 服务器配置-Token
        appid='wx72***********4704',  # 公众号开发信息-开发者ID
        appsecret='22843c*********************284b2',  # 公众号开发信息-开发者密码
        encrypt_mode='normal',  # 服务器配置-明文模式
        encoding_aes_key='1b9iW6t8XUKEhQkVfgVlPqjawX23qCq9oYYvVLUUDvB'  # 服务器配置-EncodingAESKey
    )
    signature = request.GET.get('signature')  # 获取请求信息
    timestamp = request.GET.get('timestamp')
    nonce = request.GET.get('nonce')
    wechat_instance = WechatBasic(conf=conf)  # 实例化微信基类对象
    if not wechat_instance.check_signature(signature=signature, timestamp=timestamp, nonce=nonce):  # 检查验证请求的签名
        return HttpResponseBadRequest('Verify Failed')
    else:
        if request.method == 'GET':
            return HttpResponse(request.GET['echostr'], None)  # 返回


def get_sign_key():  # 获取沙箱签名的函数
    '''
        本函数中的使用的参数和密钥数据均为虚构。
    '''
    params = {'mch_id': '14******32', 'nonce_str': 'ibuaiVcKdpRxkhJA'}
    url = 'https://api.mch.weixin.qq.com/sandboxnew/pay/getsignkey'  # 接口地址
    mch_key = '7FCAAD*********************0FF7A'
    sign = get_sign(params, mch_key)  # 完成加密并转为大写
    params.setdefault('sign', sign)  # 参数字典增加签名
    xml = trans_dict_to_xml(params)  # 转换字典为XML
    response = requests.request('post', url, data=xml)  # 以POST方式向微信公众平台服务器发起请求
    print(response.content.decode())
    return response.content  # 返回响应结果


def trans_dict_to_xml(data_dict):  # 定义字典转XML的函数
    data_xml = []
    for k in sorted(data_dict.keys()):  # 遍历字典排序后的key
        v = data_dict.get(k)  # 取出字典中key对应的value
        if k == 'detail' and not v.startswith('<![CDATA['):  # 添加XML标记
            v = '<![CDATA[{}]]>'.format(v)
        data_xml.append('<{key}>{value}</{key}>'.format(key=k, value=v))
    return '<xml>{}</xml>'.format(''.join(data_xml))  # 返回XML


def trans_xml_to_dict(data_xml):
    soup = BeautifulSoup(data_xml, features='xml')
    xml = soup.find('xml')  # 解析XML
    if not xml:
        return {}
    data_dict = dict([(item.name, item.text) for item in xml.find_all()])
    return data_dict


def wxpay(request):
    url = 'https://api.mch.weixin.qq.com/sandboxnew/pay/unifiedorder'  # 微信扫码支付接口
    key = 'c4966*********************069097'  # 沙箱交易密钥
    total_fee = int(float(request.GET['price']) * int(request.GET['quantity']) * 100)  # 订单总金额（注意是分不是元）
    body = request.GET.get('goods_name').encode()  # 商品描述
    out_trade_no = 'order_%s' % random.randrange(100000, 999999)  # 订单编号
    params = {
        'appid': 'wx77**********9aab',  # 沙箱环境的APPID
        'mch_id': '14******32',  # 商户号
        'notify_url': 'http://d0a10b0e.ngrok.io/check_wxpay/',  # 回调地址
        'product_id': 'goods_%s' % random.randrange(100000, 999999),  # 商品编号
        'trade_type': 'NATIVE',  # 支付类型（扫码支付）
        'spbill_create_ip': '1.203.76.199',  # 发送请求服务器的IP地址
        'total_fee': total_fee,  # 订单总金额
        'out_trade_no': out_trade_no,  # 订单编号
        'body': body,  # 商品描述
        'nonce_str': 'ibuaiVcKdpRxkhJA'  # 字符串
    }
    sign = get_sign(params, key)  # 获取签名
    params.setdefault('sign', sign)  # 添加签名到参数字典
    xml = trans_dict_to_xml(params)  # 转换字典为XML
    response = requests.request('post', url, data=xml)  # 以POST方式向微信公众平台服务器发起请求
    data_dict = trans_xml_to_dict(response.content)  # 将请求返回的数据转为字典
    qrcode_name = out_trade_no + '.png'  # 支付二维码图片保存路径
    if data_dict.get('return_code') == 'SUCCESS':  # 如果请求成功
        img = qrcode.make(data_dict.get('code_url'))  # 创建支付二维码片
        img.save(settings.STATIC_ROOT + '/' + qrcode_name)  # 保存支付二维码
        return render(request, 'qrcode.html', {'qrcode_img': qrcode_name})  # 为支付页面模板传入二维码图像
    return HttpResponse('交易请求失败！')


@csrf_exempt  # 去除csrf验证
def check_wxpay(request):
    data_dict = trans_xml_to_dict(request.body)  # 回调数据转字典
    sign = data_dict.pop('sign')  # 取出签名
    key = 'c4966*********************069097'  # 商户交易密钥
    back_sign = get_sign(data_dict, key)  # 计算签名
    if sign == back_sign:  # 验证签名是否与回调签名相同
        '''
        检查对应业务数据的状态，判断该通知是否已经处理过，如果没有处理过再进行处理，如果处理过直接返回结果成功。
        '''
        print('支付成功！')
        return HttpResponse('SUCCESS')
    else:
        '''
            此处编写支付失败后的业务逻辑
        '''
        return HttpResponse('')


def get_sign(data_dict, key):  # 签名函数，参数为签名的数据和密钥
    params_list = sorted(data_dict.items(), key=lambda e: e[0], reverse=False)  # 参数字典倒排序为列表
    params_str = "&".join(u"{}={}".format(k, v) for k, v in params_list) + '&key=' + key
    # 组织参数字符串并在末尾添加商户交易密钥
    md5 = hashlib.md5()  # 使用MD5加密模式
    md5.update(params_str.encode())  # 将参数字符串传入
    sign = md5.hexdigest().upper()  # 完成加密并转为大写
    return sign
